// src/SlugTerm.hh
// This file is part of SlugTerm; see http://chezphil.org/slugterm
// (C) 2005 Philip Endecott
// Modifications for 4D Systems Display Graphics Modules (C) 2008 Sven Killig <sven@killig.de>

// This is version $Name:  $
//   (if there is no version (e.g. V0-1) mentioned in the previous line,
//    this is probably a snapshot from between "official" releases.)

// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#ifndef SlugTerm_hh
#define SlugTerm_hh

#include "UsbKeyboard.hh"
#include "UsbLcd.hh"
#include "Terminal.hh"

#include <boost/thread/mutex.hpp>
#include <boost/thread/condition.hpp>
#include <boost/multi_array.hpp>

#include <string>
#include <vector>


class SlugTerm {

public:
  typedef std::vector<std::string> commands_t;

  SlugTerm(std::string event_dev_fn,
	   KeyboardLayout& keyboard_layout,
	   std::string device, int bps, int font,
	   commands_t commands,
	   int screenblank_delay=60);
  ~SlugTerm();

  void run(void);

private:
  UsbKeyboard keyboard;
  UsbLcd lcd;
  int screenblank_delay_;
  typedef std::vector<Terminal*> terms_t;
  terms_t terms;
  int current_term;

  bool pending_changes;
  boost::mutex pending_changes_mutex;
  boost::condition pending_changes_condition;

  typedef boost::multi_array<unsigned char,2> screen_t;
  screen_t old_screen,old_screen_attr;

  void process_keys(std::string keys);
  void note_term_changed(void);
  void update_lcd(void);
};


#endif
