// src/Terminal.cc
// This file is part of SlugTerm; see http://chezphil.org/slugterm
// (C) 2005 Philip Endecott
// Modifications for 4D Systems Display Graphics Modules (C) 2008 Sven Killig <sven@killig.de>

// This is version $Name:  $
//   (if there is no version (e.g. V0-1) mentioned in the previous line,
//    this is probably a snapshot from between "official" releases.)

// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#include "Terminal.hh"

#include <string>
#include <boost/bind.hpp>
#include <boost/function.hpp>
#include <boost/thread/mutex.hpp>

#include "Exception.hh"
#include "select.hh"

using namespace std;


Terminal::Terminal(int rows,
		   int cols,
		   string command,
		   boost::function<void(void)> onChange
		   ):
  rt(rote_vt_create(rows,cols)),
//  mutex(boost::read_write_scheduling_policy::writer_priority),
  onChange(onChange)
{
  pid_t rc = rote_vt_forkpty(rt,command.c_str());
  if (rc==-1) {
///    throw SysException("rote_vt_forkpty("+command+")");
  }
}


Terminal::~Terminal()
{
  rote_vt_destroy(rt);
}


int Terminal::rows(void) const
{
  return rt->rows;
}

int Terminal::cols(void) const
{
  return rt->cols;
}


RoteCell Terminal::cell(int r, int c) const
{
  return rt->cells[r][c];
}


int Terminal::cur_row(void) const
{
  return rt->crow;
}


int Terminal::cur_col(void) const
{
  return rt->ccol;
}


bool Terminal::curpos_dirty(void) const
{
  return rt->curpos_dirty;
}


bool Terminal::line_dirty(int line) const
{
  return rt->line_dirty[line];
}

void Terminal::line_dirty_set(int line, bool dirty) const
{
  rt->line_dirty[line] = dirty;
}


void Terminal::write(string data)
{
  write_lock_t l(mutex);
  rote_vt_write(rt, data.data(), data.length());
}


void Terminal::run(void)
{
  while (1) {
    select_r(rote_vt_get_pty_fd(rt));
    {
      write_lock_t l(mutex);
      rote_vt_update(rt);
    }
    onChange();
  }
}

