// src/UsbLcd.cc
// This file is part of SlugTerm; see http://chezphil.org/slugterm
// (C) 2005 Philip Endecott
// Modifications for 4D Systems Display Graphics Modules (C) 2008 Sven Killig <sven@killig.de>

// This is version $Name:  $
//   (if there is no version (e.g. V0-1) mentioned in the previous line,
//    this is probably a snapshot from between "official" releases.)

// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#include "UsbLcd.hh"

#include "4D.h"

#include <stdint.h>
#include <unistd.h>

#include <string>
#include <vector>

#include <boost/assign/std/vector.hpp>

using namespace std;
using namespace boost::assign;

_4DModule uoled128gmd1;
var_screeninfo	myscreen;
int XRES, YRES, FONT;

UsbLcd::UsbLcd(string device, int bps, int font)
{
  //send(ir,0x22);
  //send(ir,0x28);

  //uoled128gmd1.modconfig("/dev/ttyUSB0", 230400);
  //uoled128gmd1.modconfig("/dev/XPort", 230400);
  uoled128gmd1.modconfig(device.c_str(), bps);
  uoled128gmd1.modup();
backlight(true);
  myscreen=uoled128gmd1.getScreenInfo();
//fprintf(stderr,"%s", myscreen.id);
fprintf(stderr,"%ix%i\n", myscreen.xres, myscreen.yres);
  uoled128gmd1.setContrast(15);
  FONT=font;
  uoled128gmd1.setFontSize(FONT);
  switch(FONT) {
    case 0: XRES= 6; YRES= 8; break;
    case 1: XRES= 8; YRES= 8; break;
    case 2: XRES= 8; YRES=12; break;
    case 3: XRES=12; YRES=16; break;
  }
  setOpacity(0);
  uoled128gmd1.setPenSize(0);
  uoled128gmd1.setTimeout(500);
  //uoled128gmd1.writeWaitForACK(0);
  ///define chars...

  clear();

  set_mode(true,true,false);
}


UsbLcd::~UsbLcd()
{
}


void UsbLcd::clear(void)
{
  //send(ir,0x01,100000);
  uoled128gmd1.eraseScreen();
  current_row=0;
  current_col=0;
  current_pos_known=true;
}


void UsbLcd::home(void)
{
  //send(ir,0x02,1520);
  current_row=0;
  current_col=0;
  current_pos_known=true;
}


bool display_,line_cursor_;
void UsbLcd::set_mode(bool enable_display,
		      bool enable_line_cursor,
		      bool enable_block_cursor)
{
//fprintf(stderr,"set_mode(%i, %i, %i)\n", enable_display, enable_line_cursor, enable_block_cursor);
  /*send(ir,
       0x8 | (enable_display?0x4:0) | (enable_line_cursor?0x2:0)
       | (enable_block_cursor?0x1:0) );*/
  /*if(display_!=enable_display) {
    display_=enable_display;
    // happens in backlight():
    if(enable_display) uoled128gmd1.powerUp();
    else uoled128gmd1.moddown();
  }*/
  if(line_cursor_!=enable_line_cursor) {
    line_cursor_=enable_line_cursor;
    int color;
    if(enable_line_cursor) color=65535;
    else color=0;
    uoled128gmd1.drawLine(current_col*XRES,(current_row+1)*YRES-1,(current_col+1)*XRES-1,(current_row+1)*YRES-1,color);
  }
  //uoled128gmd1.setDisplay(enable_display);
}


void UsbLcd::set_pos(int row, int col)
{
//fprintf(stderr,"set_pos(%i, %i)\n", row, col);
  if (current_pos_known
      && (current_row==row)
      && (current_col==col)) {
    return;
  }
  current_row=row;
  current_col=col;
  current_pos_known=true;
}


void UsbLcd::write(char c, unsigned char attr)
{
//std::cout << c;
  //send(dr,latin1_to_hd44780A00(c));
  int r=0,g=0,b=0,i;
  int gc[2],gc16[2];
  gc[0]=ROTE_ATTR_BG(attr);
  gc[1]=ROTE_ATTR_FG(attr);
  for (i = 0; i <= 1; i++) {
      switch(gc[i]) {
        case 0: r= 0; g= 0; b= 0; break; // black
        case 1: r=31; g= 0; b= 0; break; // red
        case 2: r= 0; g=63; b= 0; break; // green
        case 3: r=31; g=63; b= 0; break; // yellow
        case 4: r= 0; g= 0; b=31; break; // blue
        case 5: r=31; g= 0; b=31; break; // magenta
        case 6: r= 0; g=63; b=31; break; // cyan
        case 7: r=31; g=63; b=31; break; // white
      }
      if(!i || (i && !(ROTE_ATTR_BOLD(attr) || ROTE_ATTR_BLINK(attr)))) {
        r>>=1;
        g>>=1;
        b>>=1;
      }
      gc16[i]=RGB_to16(r, g, b);
  }
  uoled128gmd1.drawRectangle(current_col*XRES,current_row*YRES,(current_col+1)*XRES-1,(current_row+1)*YRES-1,gc16[0]);
  uoled128gmd1.placeText(c,current_col,current_row,gc16[1]);
  current_col++;
  if (current_col>=cols()) {
    current_pos_known=false;
  }
}

void UsbLcd::write(char c)
{
  write(c,0x70);
}

bool o_;
void UsbLcd::setOpacity(bool o)
{
  if (o_!=o) {
    o_=o;
    uoled128gmd1.setTextOpacity(o);
  }
}

void UsbLcd::write(string s, unsigned char attr)
{
//std::cout << s;
  /*for(unsigned int i = 0; i<s.length(); ++i) {
    send(dr,latin1_to_hd44780A00(s[i]));
    current_col++;
  }*/
  int r=0,g=0,b=0,i;
  int gc[2],gc16[2];
  gc[0]=ROTE_ATTR_BG(attr);
  gc[1]=ROTE_ATTR_FG(attr);
  for (i = 0; i <= 1; i++) {
      switch(gc[i]) {
        case 0: r= 0; g= 0; b= 0; break; // black
        case 1: r=31; g= 0; b= 0; break; // red
        case 2: r= 0; g=63; b= 0; break; // green
        case 3: r=31; g=63; b= 0; break; // yellow
        case 4: r= 0; g= 0; b=31; break; // blue
        case 5: r=31; g= 0; b=31; break; // magenta
        case 6: r= 0; g=63; b=31; break; // cyan
        case 7: r=31; g=63; b=31; break; // white
      }
      if(!i || (i && !(ROTE_ATTR_BOLD(attr)/* || ROTE_ATTR_BLINK(attr)*/))) {
        r>>=1;
        g>>=1;
        b>>=1;
      }
      gc16[i]=RGB_to16(r, g, b);
  }
  if(gc16[0]==0)
    setOpacity(true);
  else {
    setOpacity(false);
    uoled128gmd1.drawRectangle(current_col*XRES,current_row*YRES,(current_col+s.length())*XRES-1,(current_row+1)*YRES-1,gc16[0]);
  }
  string s2="";
  for(int i=0;i<s.length();i++) {
    unsigned char c=s.at(i);
    if(c>127) {
      if     (c==0xc4 || c==0xcd)                                             s2+="-";
      else if(c==0xb3)                                                        s2+="|";
      else if(c==0xb4 || c>=0xbf && c<=0xc3 || c==0xc5 || c>=0xd9 && c<=0xda || c==0xc9) s2+="+";
      else fprintf(stderr,"%02X('%c') ", c, c);
    }
    else s2+=c;
  }
  uoled128gmd1.placeString((char)current_col,(char)current_row,(char)FONT,(short int)gc16[1],&s2);
  if(ROTE_ATTR_BLINK(attr)) {
    uoled128gmd1.drawLine(current_col*XRES,(current_row+1)*YRES-1,(current_col+s.length())*XRES-1,(current_row+1)*YRES-1,gc16[1]);
  }
  current_col+=s.length();
  if (current_col>=cols()) {
    current_pos_known=false;
  }
}

void UsbLcd::write(string s) {
  write(s,0x70);
}

void UsbLcd::write_at(int row, int col, char ch, unsigned char attr)
{
  set_pos(row,col);
  write(ch,attr);
}

void UsbLcd::write_at(int row, int col, char ch)
{
  write_at(row,col,ch,0x70);
}


void UsbLcd::write_at(int row, int col, string s, unsigned char attr)
{
  set_pos(row,col);
  write(s,attr);
}

void UsbLcd::write_at(int row, int col, string s)
{
  write_at(row,col,s,0x70);
}


void UsbLcd::backlight(bool enable)
{
//fprintf(stderr,"backlight(%i)\n", enable);
  if (backlight_!=enable) {
    backlight_=enable;
    //ftdi.write(encode(backlight_,dr,false,false,0));
    if(enable) {
      ///uoled128gmd1.powerUp();
      ///sleep(1);
      ///uoled128gmd1.setContrast(15);
    } ///else uoled128gmd1.powerDown();
  }
}


int UsbLcd::rows(void) const
{
  return myscreen.yres/YRES;
}


int UsbLcd::cols(void) const
{
  return myscreen.xres/XRES;
}
